/**
 * @file
 * Provides GridStack Outlayer base shared utility methods.
 */

(function (Drupal, _db) {

  'use strict';

  /**
   * Outlayer utility functions.
   *
   * @namespace
   */
  Drupal.outLayer = Drupal.outLayer || {
    $el: null,
    $instance: null,
    isBlazy: false,
    isNativeGrid: false,
    isUnGridStack: false,

    /**
     * Updates grid boxes with the given aspect ratios to make them responsive.
     *
     * Basically avoids hard-coding widths and heights thanks to padding hack.
     * This is one good reason to avoid using Imagesloaded plugin since we
     * don't have to wait for any image to finish loading to have a grid layout.
     * Consider this as a frame for a picture image.
     *
     * @param {HTMLElement} items
     *   The child HTML elements.
     * @param {bool} forceHeight
     *   Whether to force element height to avoid excessive height.
     */
    updateRatio: function (items, forceHeight) {
      forceHeight = forceHeight || false;
      _db.forEach(items, function (item) {
        var width = item.hasAttribute('data-ol-width') ? parseInt(item.getAttribute('data-ol-width')) : -1;
        var height = item.hasAttribute('data-ol-height') ? parseInt(item.getAttribute('data-ol-height')) : -1;
        var pad = Math.round(((height / width) * 100), 2);
        var $media = item.querySelectorAll('.media--ratio, .b-bg');
        var $content = item.querySelector('.box__content');

        // Supports multiple media such as an embedded Slick carousel.
        if ($media !== null && height > 0 && width > 0) {
          _db.forEach($media, function (medium) {
            medium.style.paddingBottom = pad + '%';
          });
        }

        // Fixed for the new CSS Grid during filtering, sorting, searching.
        if (forceHeight && $content !== null && item.offsetHeight > 40) {
          $content.style.maxHeight = item.offsetHeight + 'px';
        }
      });
    },

    prepare: function () {
      var me = this;
      me.isNativeGrid = me.$el.classList.contains('gridstack--native');
      me.isUnGridStack = me.$el.classList.contains('ungridstack');
      me.isBlazy = Drupal.blazy && Drupal.blazy.init !== null;
    },

    cleanUp: function () {
      var me = this;

      if (me.$el !== null && me.$el !== 'undefined') {
        window.setTimeout(function () {
          me.$el.classList.remove('gridstack--packing');

          if (me.$el.classList.contains('ungridstack')) {
            me.$el.classList.remove('gridstack--enabled');
          }
        }, 100);
      }
    }

  };

}(Drupal, dBlazy));
